<?php

class PitoModel extends CI_Model {

    /**
     * Get the properties for each FromObject field like type, name, id etc. set in the second database
     * @param string $tableName the name of the FormObject and table
     */
    public function GetFormProperties($tableName) {
        $this->db_pito = $this->load->database('pito', TRUE);

        $this->db_pito->select('*');

        $this->db_pito->from('fields');
        $this->db_pito->join('objects', 'fields.objectId = objects.objectId');
        $this->db_pito->where('objects.objectName', $tableName);
        $this->db_pito->where('(`fields`.`classtype` = \'' . 'form' . '\' OR `fields`.`classtype` = \'both\')');
        //$this->db_pito->join('requirements', 'address.requirementsId = requirements.id');

        $query = $this->db_pito->get();
        $data = $query->result();


        foreach ($data as $value) {
            //            $reqId = $value->requirementsId;
            //            $value->requirements = $this->GetValidatonRules($reqId);
            $value->requirements = $this->MakeValidationString($value);
        }

        return $data;
    }

    /**
     * Gets the relations type 1:n for an object set in the second database
     * @param string $tableName the name of the object and table
     */
    public function GetRelations($tableName) {
        $this->db_pito = $this->load->database('pito', TRUE);

        $this->db_pito->select('related');
        $this->db_pito->from('fields');
        $this->db_pito->join('objects', 'fields.objectId = objects.objectId');
        $this->db_pito->where('objects.objectName', $tableName);

        $query = $this->db_pito->get();
        $data = $query->result();

        return $data;
    }

    /**
     * Gets the validation rules for a field
     * @param integer $reqId the id of the validation rules
     * @return string
     * @deprecated since version 12.04
     */
    private function GetValidatonRules($reqId) {
        $this->db_pito = $this->load->database('pito', TRUE);

        $this->db_pito->select('*');
        $this->db_pito->from('requirements');
        $this->db_pito->where('id', $reqId);

        $query = $this->db_pito->get();
        $data = $query->result();

        //String, който ще съдържа validation rules, които ще се извикат в контролера
        $requirements = '';

        //Ако няма зададени Validation rules просто ще върне празен низ
        if (!empty($data[0])) {
            foreach ($data[0] as $key => $value) {
                if ($key == 'id') {
                    continue;
                }
                if ($key == 'min_length' || $key == 'max_length' || $key == 'exact_length' || $key == 'unique') {
                    if (!empty($value)) {
                        $requirements .= $key . "[$value]|";
                    }
                } else {
                    if ($value == 1) {
                        $requirements .= $key . '|';
                    }
                }
            }
            $requirements = substr_replace($requirements, "", -1);
            return $requirements;
        } else {
            return $requirements;
        }
    }

    private function MakeValidationString($object) {
        //Списък с всички пропъртита, които нямат общо с валидацията
        $otherPropertiesArray = array('id', 'name', 'type', 'label', 'requirementsId', 'related', 'objectId', 'classtype', 'required', 'objectName');

        //Низовият литерал, в който ще се записват валидациите
        $requirements = '';

        foreach ($object as $key => $value) {
            //Ако този елемент не се отнася до валидацията
            if (in_array($key, $otherPropertiesArray)) {
                continue;
            } else {
                if ($key == 'min_length' || $key == 'max_length' || $key == 'exact_length' || $key == 'unique') {
                    if (!empty($value)) {
                        $requirements .= $key . "[$value]|";
                    }
                } else {
                    if ($value == 1) {
                        $requirements .= $key . '|';
                    }
                }
            }
        }
        $requirements = substr_replace($requirements, "", -1);
        return $requirements;
    }

    /**
     * Взима всички стойности за обекта
     * @param String $table Име на таблица, в която да се търсят стойностите
     * @param integer $id    Id на записа
     */
    public function GetObject($table, $id) {
        $this->db->select('*');
        $this->db->from($table);
        $this->db->where($table . 'Id', $id);

        $query = $this->db->get();
        $data = $query->result();

        $field_data = $this->db->field_data($table);

        foreach ($field_data as $field) {
            $key = $field->name;

            //Взимат се данните за полетата на таблицата
            //После се прави проверка, ако информацията е число 
            //и в базата е записано като такова, тогава парсва стринга
            if (isset($data[0])) {
                if ((is_numeric($data[0]->$key)) && $field->type != 'varchar') {
                    $data[0]->$key = (double) $data[0]->$key;
                }
                //Ако е булева стойност се връща стойността true или false 
                //вместо репрезентиращ низов литерал
                if ($field->type == 'boolean') {
                    $data[0]->$key = (bool) $data[0]->$key;
                }
            }
        }

        if (empty($data)) {
            return null;
        } else {
            return $data[0];
        }
    }

    /**
     * Gets the name relatedt able. The realation is of type 1:1
     * @param integer $id the id of the table to which the other table is related
     */
    public function GetRelatedTableName($id) {
        $this->db_pito = $this->load->database('pito', TRUE);

        $this->db_pito->select('objectName');
        $this->db_pito->from('objects');
        $this->db_pito->where('objectId', $id);

        $query = $this->db_pito->get();
        $data = $query->result();

        return $data[0]->objectName;
    }

    /**
     * Searches for records in the table that match a certain criteria
     * @param string $table       the name of the table
     * @param Array $searchArray  the criteria that must be satisfied. The format sould be [name of field] => [value it should be equal to]
     */
    public function FindByCriteria($table, $searchArray) {
        $this->db = $this->load->database('default', TRUE);

        $this->db->select('*');
        $this->db->from($table);

        foreach ($searchArray as $key => $value) {
            if ($this->db->field_exists($key, $table)) {
                $this->db->where($key, $value);
            } else {
                throw new Exception("the field $key does not exist in $table");
            }
        }

        $query = $this->db->get();
        $data = $query->result();
        //        var_dump($this->db->last_query());
        return $data;
    }

    /**
     * Searches usind an array of arrays which holds the search criterias
     * @param string $table       the name of the table to be searched
     * @param Array $searchArray an array of arrays, each one holdind a 
     * column => value to be seached as a first element and 'operator' => operator 
     * (like, <, =, != etc.) as a second element
     */
    public function FindByDetailedCriteria($table, $searchArray, $sortArray = array(), $page = '', $limit = '') {
        //echo 'tuk';

        $this->db->select('*');
        $this->db->from($table);
        if (count($sortArray) > 0) {
            foreach ($sortArray as $sort) {
                $this->db->order_by($sort->column, $sort->direction);
            }
        }
        if ($limit != "" && $page == '') {
            $this->db->limit($limit);
        }
        if ($page != '') {
            $offset = $page * $limit;
            $this->db->limit($limit, $offset);
        }
        foreach ($searchArray as $searchCriteria) {
            //            var_dump($searchCriteria);
            //$searchCriteria е всеки един масив, който е в $searchArray. В $searchCriteria 
            //може да има четири стойности - column, value, operator и table, като последната 
            //се използва ако трябва да има join с друга таблица. Заявката се създава от всеки един $searchCriteria масив 
            $queryString = $this->makeQueryString($searchCriteria);
            var_dump($searchCriteria);
            if (isset($searchCriteria['or']) && is_array($searchCriteria['or'])) {
                 var_dump("Ura");
          
                $query = "($queryString ";
                if (isset($searchCriteria['or'][0])) {
                    foreach ($searchCriteria['or'] as $orCriteria) {
                        $orQuery = $this->makeQueryString($orCriteria);
                        $query .= "OR $orQuery ";
                    }
                    $query .= ")";
                }else{
                    $orQuery = $this->makeQueryString($searchCriteria['or']);
                    $query = "($queryString OR $orQuery)";
                }

                
                //$this->db-where($query);
            } else {
                $query = $queryString;
            }
            $this->db->where($query);
            //Ако масивът има стойност в ключа table тогава трябва да се направи join с друга таблица
            //Предполага се, че търсенето ще бъде от рода на city, city.cityId = address.citytId
            if (!empty($searchCriteria['table'])) {
                $joinTable = $searchCriteria['table'];
                $this->db->join($joinTable, "$joinTable.$joinTable" . 'Id = ' . "$table.$joinTable" . 'Id');
            }
        }

        $query = $this->db->get();
        $result = $query->result();
        $data[0] = $this->db->last_query();
        //     ob_clean();
       // echo $this->db->last_query();
        //die;
        //        var_dump($this->db); 
        //          var_dump($data);
        //        die; 
        foreach ($result as $obj) {
            if (isset($obj->paintingID)) {
                $obj->paintingID = (int) $obj->paintingID;
            }
            $data[1][] = $obj;
        }
        // echo '<pre>'; print_r($data); echo '</pre>'; die;
        return $data;
    }

    /**
     * Sets a record as deleted
     * @param string $table the name of the object's table
     * @param integer $id   the object's is
     */
    protected function DeleteRecord($table, $id) {
        $update_array = array(
            'deleted' => 1
        );

        $this->db->where($table . 'Id', $id);
        $this->db->update($table, $update_array);
    }

    /**
     * Searches for a connection type 1:n
     * @param string $tableName   the table of the related objects
     * @param string $parameter   the column by which tehay are related to the object
     * @param integer $parameterId the id of the object
     */
    public function FindRelatedRecords($tableName, $parameter, $parameterId) {
        if ($this->db->table_exists($tableName) && $this->db->field_exists($parameter, $tableName)) {
            $this->db->select('*');
            $this->db->from($tableName);
            $this->db->where($parameter, $parameterId);

            $query = $this->db->get();
            $data = $query->result();

            return $data;
        } else {
            return '';
        }
    }

    /**
     * Creates or updates an entity
     * @param array $arrayToInsert the values that will be inserted into the table
     * @param string $table the name of the table
     * @param int $objectId the object's Id
     * @return boolean, int
     */
    public function SaveObject($arrayToInsert, $table, $objectId) {
        $objectIdName = $table . "Id";

        if (!empty($objectId)) {
            if (array_key_exists('editDate', $arrayToInsert)) {
                $arrayToInsert['editDate'] = date('Y-m-d H:i:s');
            }
            $this->db->where($objectIdName, $objectId);
            $this->db->update($table, $arrayToInsert);
            return true;
        } else {
            if (array_key_exists('createDate', $arrayToInsert)) {
                $arrayToInsert['createDate'] = date('Y-m-d H:i:s');
            }
            $this->db->insert($table, $arrayToInsert);
            return $this->db->insert_id();
        }
    }

    public function makeQueryString($searchCriteria) {
        //Проверка за всеки един масив какъв оператор е подаден

        switch (strtolower($searchCriteria['operator'])) {
            case 'like':
                $whereString = $searchCriteria['column'] . ' like "' . $searchCriteria['value'] . '%"';
                break;
            case '=':
            case '==':
            case 'equal':
            case 'is':
                $whereString = $searchCriteria['column'] . " = '" . $searchCriteria['value'] . "'";
                break;
            case '>':
            case 'greater than':
                $whereString = $searchCriteria['column'] . " >'" . $searchCriteria['value'] . "'";
                break;
            case '<':
            case 'less than':
                $whereString = $searchCriteria['column'] . " <'" . $searchCriteria['value'] . "'";
                break;
            case '!=':
            case 'not equal':
                $whereString = $searchCriteria['column'] . " !='" . $searchCriteria['value'] . "'";
                break;
            case '>=':
            case 'greater or equal':
                $whereString = $searchCriteria['column'] . " >='" . $searchCriteria['value'] . "'";
                break;
            case '<=':
            case 'less or equal':
                $whereString = $searchCriteria['column'] . " <='" . $searchCriteria['value'] . "'";
                break;
            case 'null';
                $whereString = $searchCriteria['column'] . ' IS NULL';
                break;
            case 'in';
                $whereString = $searchCriteria['column'] . " IN'" . $searchCriteria['value'] . "'";
                break;
            default:
                $whereString = $searchCriteria['column'] . " = '" . $searchCriteria['value'] . "'";
                break;
        }

        return $whereString;
    }

}
