<?php
require_once('application/pito_class/Messages.php');

abstract class PitoMain {
    /**
     * Gets the form properties, validation strings and relations of an object
     * @param string $tableName name of the table and object
     */
    public function GetFormAndRequirements ($tableName) {
        $ci =& get_instance();
        $ci->load->model('PitoModel');
        $tableName = strtolower($tableName);
        $formProperties = $ci->PitoModel->GetFormProperties($tableName);
        
        return $formProperties;
    }
    
    /**
     * Returns the related objects if there are any
     * @param string $tableName the name of the object and table
     */
    protected function GetRelated($tableName) {
        $ci =& get_instance();
        $ci->load->model('PitoModel');
        $tableName = strtolower($tableName);
        $formProperties = $ci->PitoModel->GetRelations($tableName);
        
        return $formProperties;
    }

    /**
     * Gets an object from the table   
     * @param string $table table and object name
     * @param intager $id   object's id
     */
    protected function GetObjectById($table, $id) {
        $ci =& get_instance();
        $ci->load->model('PitoModel');
        
        $table = strtolower($table);
        $objectValues = $ci->PitoModel->GetObject($table, $id);
        
        return $objectValues;
    }
    
    protected function GetRelatedTableName($id) {
        $ci =& get_instance();
        $ci->load->model('PitoModel');
        $relatedTableName = $ci->PitoModel->GetRelatedTableName($id);
        
        return $relatedTableName;
    }
    
    /**
     * Assign an object to another object's field that represents an 1:1 connection
     * @param array $fields the object's properties
     */
    protected function AssignFields($fields, $nestingLevel=0) {
        foreach ($fields as $value) {
            $id = $value->related;   
            if (!empty($id)) {
                //ако някоя от променливите трябва да съхранява друг обект (т.е. имаме релация)
                //тогава $tempName съдържа типа на обекта с малки букви
                $tempName = $this->GetRelatedTableName($id);
                //$propertyName съхранява името на променливата
                $propertyName = $value->name;
                $relatedObjectName = ucfirst($tempName);
                $relatedCustomName = ucfirst($tempName);
                
                $relatedObjectName .= 'Object';
                $relatedCustomName .= 'Custom';
                
                //Ако propertie-то се намира в масива, тогава името му от базата не отговаря на това в обекта. 
                //Името в базата е ключът на масива, докато името му в обекта е стойността. 
                //Заменя се полученото в $propertyName име с това на член-данната от обекта и се инициализира 
                
//                if (key_exists($propertyName, $propertiesArray)) {
//                    $propertyName = $propertiesArray[$propertyName];
//                }
                
                if (class_exists($relatedCustomName)) {
                    if ($nestingLevel > 0) {
                        $newNestingLevel = $nestingLevel;
                        $newNestingLevel--;
                        //$relatedCustomName съхранява името на този custom, създаден от потребителя клас, който трябва да се инстанцира
                        $this->$propertyName = new $relatedCustomName($newNestingLevel);
                    }
                } 
                else {
                    if (class_exists($relatedObjectName)) {
                        if ($nestingLevel > 0) {
                            $newNestingLevel = $nestingLevel;
                            $newNestingLevel--;
                            //$reladedObjectName съхранява името на този обект, който трябва да се създаде
                            $this->$propertyName = new $relatedObjectName($newNestingLevel);
                        }
                    } 
                    else {
                        throw new Exception('No such class: ' . $relatedObjectName);
                    }
                }
            }
        }
    }

    //Търси в таблицата стойностите, подадени в масива като ключът е колоната, стойността е критерият, по който се търси.
    //Nesting Level указва колко вложени обекти да има във върнатият резултат
    protected static function Search($table, $searchArray, $nestingLevel=0,$sortArray=array(),$page = '',$limit='',$customModel='') {

        $ci =& get_instance();
        $ci->load->model('PitoModel');
	if(!is_array($searchArray))
	{
	    $searchArray = $nestingLevel;
	}
	    $firstKey = key($searchArray);

        if (is_array($searchArray[$firstKey]) && array_key_exists('operator', $searchArray[$firstKey])) {
            $result = $ci->PitoModel->FindByDetailedCriteria(strtolower($table), $searchArray,$sortArray,$page,$limit);
        }
        else {
            $result = $ci->PitoModel->FindByCriteria(strtolower($table), $searchArray);
        }
        return $result;

        $resultArray = array();
        $className = ucfirst($table);
        $customClassName = ucfirst($table);

        $className .= 'Object';
        $customClassName .= 'Custom';
//var_dump($result);
        foreach ($result as $object) {
	    
            if (class_exists($customModel)) {
                $tempVar = new $customClassName;
                $idName = $table . 'ID';

                $tempVar->LoadElement($object->$idName, $nestingLevel);
            } 
            else if (class_exists($customClassName)) {
                $tempVar = new $customClassName;
                $idName = $table . 'ID';

                $tempVar->LoadElement($object->$idName, $nestingLevel);
            } 
            else {
                if (class_exists($className)) {
                    $tempVar = new $className;
                    $idName = $table . 'ID';
                    $tempVar->LoadElement($object->$idName, $nestingLevel);
                      //die;
                } 
                else {
                    throw new Exception('No such class: ' . $className);
                }
            }
            array_push($resultArray, $tempVar);
        }
        return $resultArray;
    }

    /**
     * Fill the properties of the object with the received values from the POST form
     * @param string $prefix if the form fieds have a prefix before their name 
     */
    public function FillDataFromPost($prefix='') {
        $prefixLenght = strlen($prefix);

        foreach ($_POST as $key => $value) {
            //Прави се проверка дали има POST ключ с подаденият префикст (търси за property-та от рода на test-cityId)
            if ($prefix == substr($key, 0, $prefixLenght)) {

                //Името на property-то ще бъде $key без префикса (propery cityId ще вземе стойност от test-cityId)
                $property = substr($key, $prefixLenght);

                if (property_exists($this, $property)) {
                    $this->$property = $value;
                }
            }
        }
    }

    /**
     * Returns an array of linked elements. 
     * @param string  $firstTable               The name of the linked table
     * @param string  $secondTable              The name of the object's table. This abject will have a property that holds the returned array
     * @param integer  $id                      The object's Id
     * @param integer $relatedArrayNestingLevel The nesting og the elements in the returned array
     */
    protected function FindRelatedRecords($firstTable, $secondTable, $id, $relatedArrayNestingLevel=0) {
        $firstTable = strtolower($firstTable);
        $secondTable = strtolower($secondTable);
        
        $ci =& get_instance();
        $ci->load->model('PitoModel');

        $result = $ci->PitoModel->FindRelatedRecords($firstTable.'to'.$secondTable, $secondTable.'Id', $id);

        //В $returnedArray се записват всички обекти, които имат релация 1:n или n:m
        $returnedArray = array();
        //Записва се името на класа, което е името на таблицата, записана като CamelCase
        $className = ucfirst($firstTable);
        $customClassName = ucfirst($firstTable);
        
        $className .= 'Object';
        $customClassName .= 'Custom';
        //Името на колоната, която съхранява Id-то
        $idName = $firstTable . 'Id';

        //Ако не са намерени елементи в таблицата [име]to[име2] ще се търси в [име2]to[име]
        if (empty($result)) {
            $className = ucfirst($secondTable);
            $customClassName = ucfirst($secondTable);
            
            $className .= 'Object';
            $customClassName .= 'Custom';
            $idName = $secondTable . 'Id';

            $result = $ci->PitoModel->FindRelatedRecords($secondTable.'to'.$firstTable, $secondTable.'Id', $id);
        }

        if (!empty($result)) {
            //$value съхранява релацията. Името на първата таблица със суфикс Id, ще даде Id-то на записа. 
            //Самото property се казва с името на първата таблица + Id. $value->$idName съхранява стойността на iD-то като число
            foreach ($result as $value) {
                //Ако съществува custom зададен клас да не се използва генерирания
                if (class_exists($customClassName)) {
                    $newRelatedObject = new $customClassName();
                }
                else {
                    $newRelatedObject = new $className();
                }
                
                $newRelatedObject->LoadElement($value->$idName, $relatedArrayNestingLevel);

                array_push($returnedArray, $newRelatedObject);
            }

            return $returnedArray;
        }
        //Ако не са намерени резултати в двете търсения тогава се предполага, че връзката е 1:n 
        //и се търси във $firstTable таблица ключ с име $secondTable.'Id', който да има стойността на $id
        //Т.е. търси се в таблица със записи за всички, които принадлежат на обекта, от който сме стартирали самото търсене
        else {
            try {
                //Връща се масив, който съдържа обекти, които сърханяват стойностите на записите
                $searchCriteria = array($secondTable . 'Id' => $id);
                $searchResults = $ci->PitoModel->FindByCriteria($firstTable, $searchCriteria);
                //Име на property, което съхранява Id-то, по което да се създаде нов обект
                $idName = $firstTable . 'Id';
                //Задаваме името на класа да бъде името на първата подадена таблица
                $className = ucfirst($firstTable);
                $customClassName = ucfirst($firstTable);
                
                $className .= 'Object';
                $customClassName .= 'Custom';

                foreach ($searchResults as $key => $value) {
                    if (class_exists($customClassName)) {
                        $newRelatedObject = new $customClassName();
                    } 
                    else {
                        if (class_exists($className)) {
                            $newRelatedObject = new $className();
                            $newRelatedObject->LoadElement($value->$idName, $relatedArrayNestingLevel);

                            array_push($returnedArray, $newRelatedObject);
                        }
                    }
                }
                return $returnedArray;
            } 
            catch (Exception $ex) {
                return null;
            }
        }

    }
    
    protected function GetPropertyClass($propName, $table) {
        $fields = self::GetFormAndRequirements($table);

        //Първо се проверява за Custom дефиниран тип
        //$className = ucfirst($propName . 'Custom');
      //  if (!class_exists($className)) {
            $className = ucfirst(strtolower($propName) . 'Object');
            
           
            //Ако и такъв тип не съществува, тогава се приема, че това е custom
            //име за property, което е обект а името на класа на обекта се пази в related таблица
            if (!class_exists($className)) {
                foreach ($fields as $field) {
                    if ($propName == $field->name) {
                        $relatedName = self::GetRelatedTableName($field->related);

                        $className = ucfirst($relatedName);
                        $className = $className . 'Custom';
                        if (!class_exists($className)) {
                            $className = ucfirst($relatedName);
                            $className = $className . 'Object';
                        }
                    }
                }
            }
    //    }
        return $className;
    }

}
